/*
 * @(#) $Id$
 *
 * Copyright (c) 1998 Third Eye Consulting, Inc.  All rights reserved.
 *
 */

import java.awt.*;
import java.awt.event.*;
import java.rmi.*;
import java.rmi.server.*;
import java.util.*;

public class LibraryClientImpl extends Frame implements LibraryClient, ActionListener, ItemListener {

  public static final int DEF_PORT = 1099;

  protected LibraryServer server = null;

  protected static final int WIDTH = 400;

  protected static final int HEIGHT = 200;

  protected Vector vBooks = null;

  protected Hashtable books = new Hashtable();

  private List bookList = null;

  private TextField title = null, author = null, publisher = null,
    date = null, status = null;

  public static void main(String args[]) {
    try {
      new LibraryClientImpl();
    }
    catch (Exception e) {
      e.printStackTrace();
      System.exit(-1);
    }
  }

  public LibraryClientImpl() {
    super("Library Client");

    connect("localhost");
    
    try {
      vBooks = server.getBooks();
    } catch (RemoteException e) {
      e.printStackTrace();
    }

    // setup remaining panels
    constructLayout();
      
    // handle window close requests
    this.addWindowListener(new WindowAdapter() {
      public void windowClosing(WindowEvent e) { close(0); }
    });
    
    this.pack();
    this.show();   
  }

  protected void connect(String sName) {
    try {
      UnicastRemoteObject.exportObject(this);
      String serverName = "//" + sName + "/LibraryServerImpl";
      server = (LibraryServer)Naming.lookup(serverName);
      server.processRequests(this);
    } catch (java.rmi.ConnectException ce) {
      System.err.println("Error: server not started");
      close(-1);
    } catch (java.rmi.ConnectIOException cioe) {
      System.err.println("Error: Cannot connect to server at" + sName);
      close(-1);
    } catch (Exception e) {
      e.printStackTrace();
      close(-1);
    }
  }
  
  private void constructLayout() {
    this.setBackground(Color.lightGray);

    this.setLayout(new BorderLayout());
    
    bookList = new List(10);
    bookList.addItemListener(this);
    try {
      setBooks(vBooks);
    } catch (RemoteException re) {
      re.printStackTrace();
    }

    Panel details = new Panel();
    details.setLayout(new GridLayout(5, 2));
    details.add(new Label("Title"));
    details.add(title = new TextField());
    details.add(new Label("Author(s)"));
    details.add(author = new TextField());
    details.add(new Label("Publisher"));
    details.add(publisher = new TextField());
    details.add(new Label("Publish Date"));
    details.add(date = new TextField());
    details.add(new Label("Status"));
    details.add(status = new TextField());

    Panel mainGrid = new Panel();
    mainGrid.setLayout(new GridLayout(1, 2));
    mainGrid.add(bookList);
    mainGrid.add(details);
    this.add("Center", mainGrid);
    
    Panel buttons = new Panel();
    buttons.setLayout(new FlowLayout(FlowLayout.CENTER));
    Button checkin = new Button("Checkin");
    checkin.setActionCommand("checkin");
    checkin.addActionListener(this);
    buttons.add(checkin);
    Button checkout = new Button("Checkout");
    checkout.setActionCommand("checkout");
    checkout.addActionListener(this);
    buttons.add(checkout);
    Button exit = new Button("Exit");
    exit.setActionCommand("exit");
    exit.addActionListener(this);
    buttons.add(exit);
    this.add("South", buttons);
  }

  public void actionPerformed(ActionEvent e) {
    String cmd = e.getActionCommand();

    if (cmd.equals("checkin")) {
      checkin();
    }
    else if (cmd.equals("checkout")) {
      checkout();
    }
    else if (cmd.equals("exit")) {
      close(0);
    }
  }

  public void itemStateChanged(ItemEvent e) {
    ItemSelectable src = e.getItemSelectable();
    if (src.equals(bookList)) {
      Book book = (Book)books.get((String)((List)src).getSelectedItem());
      title.setText(book.getTitle());
      author.setText(book.getAuthor());
      publisher.setText(book.getPublisher());
      date.setText(String.valueOf(book.getDate().getYear()));
      status.setText(book.getStatusName());
    }
  }   

  protected void checkin() {
    String title = "";
    Book book = null;
    int i = 0;
    try {
      i = bookList.getSelectedIndex();
      title = bookList.getSelectedItem();
      server.checkin(title);
      bookList.select(i);
      book = (Book)books.get(title);
      status.setText(book.getStatusName());
    } catch (RemoteException re) {
      re.printStackTrace();
    }
  }

  protected void checkout() {
    String title = "";
    Book book = null;
    int i = 0;
    try {
      i = bookList.getSelectedIndex();
      title = bookList.getSelectedItem();
      server.checkout(title);
      bookList.select(i);
      book = (Book)books.get(title);
      status.setText(book.getStatusName());
    } catch (RemoteException re) {
      re.printStackTrace();
    }
  }

  protected void close(int status) {
    try {
      if (server != null) {
        server.stopClient(this);
      }
    } catch (RemoteException re) {
      re.printStackTrace();
    }
    this.dispose();
    System.exit(status);
  }

  public void setBooks(Vector books) throws RemoteException {
    int i = 0;
    int len = books.size();
    String title = "";
    Book book = null;
    bookList.removeAll();
    this.books.clear();
    for (i = 0; i < len; i++) {
      book = (Book)books.elementAt(i);
      title = book.getTitle();
      bookList.add(title);
      this.books.put(title, book);
    }
  }

}
