/*
 * @(#) $Id$
 *
 * Copyright (c) 1998 Third Eye Consulting, Inc.  All rights reserved.
 *
 */

import java.rmi.*;
import java.rmi.registry.*;
import java.rmi.server.*;
import java.util.*;

public class LibraryServerImpl extends UnicastRemoteObject implements LibraryServer, Runnable {
  
  public static final int DEF_PORT = 1099;

  protected Hashtable clientTable = new Hashtable();

  protected Thread notifier = null;

  protected Vector vBooks = new Vector();

  protected Hashtable books = new Hashtable();

  public static void main(String[] args) {
    int port = DEF_PORT;
    try {
      String url = null;
      // Create and install the security manager
      System.setSecurityManager(new RMISecurityManager());
      
      if (args.length > 0) {
        port = Integer.parseInt(args[0]);
      }
      LocateRegistry.createRegistry(port);
      LibraryServer server = new LibraryServerImpl();
      url = "//:" + port + "/LibraryServerImpl";
      Naming.rebind(url, server);
    } catch (ExportException ee) {
      System.err.println("Error: server already started on port " + port);
      System.exit(-1);
    } catch (Exception e) {
      e.printStackTrace();
      System.exit(-1);
    }
  } // end main

  public LibraryServerImpl() throws RemoteException {
    super();

    loadBooks();
    System.out.println("Books loaded");
  }

  public synchronized void processRequests(LibraryClient client) throws RemoteException {
    LibraryClient prevEntry = (LibraryClient)clientTable.get(client);
    if (prevEntry == null) {
      clientTable.put(client, client);
    }
    if (notifier == null) {
      notifier = new Thread(this);
      notifier.start();
    }
  }

  protected synchronized void removeClient(LibraryClient client) {
    clientTable.remove(client);
    if (clientTable.isEmpty()) {
      Thread thread = notifier;
      notifier = null;
      thread.stop();
    }
  }

  public void stopClient(LibraryClient client) throws RemoteException {
    removeClient(client);
  }

  public void run() {
  }

  protected void loadBooks() {
    String title = "";
    Book book = null;

    title = "Java in a Nutshell";
    book = new Book(title,
                    "David Flanagan",
                    "O'Reilly & Associates",
                    new Date(1997, 0, 1));
    vBooks.addElement(book);
    books.put(title, book);

    title = "Java 1.2 Unleashed";
    book = new Book(title,
                    "Jamie Jaworski",
                    "Sams",
                    new Date(1998, 0, 1));
    vBooks.addElement(book);
    books.put(title, book);

    title = "Object-Oriented Modeling and Design";
    book = new Book(title,
                    "James Rumbaugh",
                    "Prentice Hall",
                    new Date(1991, 0, 1));
    vBooks.addElement(book);
    books.put(title, book);

    title = "UML Distilled";
    book = new Book(title,
                    "Martin Fowler",
                    "Addison-Wesley",
                    new Date(1997, 0, 1));
    vBooks.addElement(book);
    books.put(title, book);
  }

  public Vector getBooks() throws RemoteException {
    return vBooks;
  }

  protected void sendBooks() throws RemoteException {
    LibraryClient client = null;
    Enumeration enum = clientTable.elements();
    while (enum.hasMoreElements()) {
      client = (LibraryClient)enum.nextElement();
      client.setBooks(vBooks);
    }
  }

  public void checkin(String title) throws RemoteException {
    Book book = (Book)books.get(title);
    if (book.getStatus() != Book.CHECKED_IN) {
      book.setStatus(Book.CHECKED_IN);
      sendBooks();
    }
  }

  public void checkout(String title) throws RemoteException {
    Book book = (Book)books.get(title);
    if (book.getStatus() == Book.CHECKED_IN) {
      book.setStatus(Book.CHECKED_OUT);
      sendBooks();
    }
  }
}